/*======================================================================*\
	Unicode character conversion functions
	for systems without proper locale support
	for substitution of:
	- locale setting / info functions
	- wide character functions
	- X input functions
\*======================================================================*/
#ifndef UWC_HEADER
#define UWC_HEADER

#include <langinfo.h>
#include <wchar.h>
#include <X11/Xlib.h>

#ifndef __restrict
#define __restrict
#endif

#ifdef __cplusplus
#define UWC_EXTERN_C extern "C"
#else
#define UWC_EXTERN_C extern
#endif

/*======================================================================*\
|*			Locale functions				*|
\*======================================================================*/

/* Set and/or return the current locale.  */
UWC_EXTERN_C char * u_setlocale (int __category, __const char * __locale);


/* Return the current locale's value for ITEM.
   If ITEM is invalid, an empty string is returned.

   The string returned will not change until `u_setlocale' is called;
   it is usually in read-only memory and cannot be modified.  */
UWC_EXTERN_C char * u_nl_langinfo (nl_item __item);


/*======================================================================*\
|*			X input functions				*|
\*======================================================================*/

/* (No description in Xlib.h) */
UWC_EXTERN_C int u_XwcLookupString (XIC ic, XKeyPressedEvent * event, 
			wchar_t * buffer_return, int wchars_buffer, 
			KeySym * keysym_return, Status * status_return);

#ifdef UWC_REPLACE_XLOOKUPSTRING
#include <X11/Xutil.h>
/* (No description in Xutil.h) */
UWC_EXTERN_C int u_XLookupString (XKeyEvent * event, 
			char * buffer_return, int bytes_buffer, 
			KeySym * keysym_return, XComposeStatus * status_in_out);
#endif /* UWC_REPLACE_XLOOKUPSTRING */

/*======================================================================*\
|*			Conversion functions				*|
\*======================================================================*/

/* Return the length of the multibyte character
   in S, which is no longer than N.  */
UWC_EXTERN_C int u_mblen (__const char * __s, size_t __n);

/* Return the length of the given multibyte character,
   putting its `wchar_t' representation in *PWC.  */
UWC_EXTERN_C int u_mbtowc (wchar_t * __restrict __pwc,
		   __const char * __restrict __s, size_t __n);

/* Put the multibyte character represented
   by WCHAR in S, returning its length.  */
UWC_EXTERN_C int u_wctomb (char * __s, wchar_t __wchar);


/* Convert a multibyte string to a wide char string.  */
UWC_EXTERN_C size_t u_mbstowcs (wchar_t * __restrict  __pwcs,
			__const char * __restrict __s, size_t __n);

/* Convert a wide char string to multibyte string.  */
UWC_EXTERN_C size_t u_wcstombs (char * __restrict __s,
			__const wchar_t * __restrict __pwcs, size_t __n);


/* Determine whether PS points to an object representing the initial
   state.  */
UWC_EXTERN_C int u_mbsinit (__const mbstate_t * __ps);

/* Write wide character representation of multibyte character pointed
   to by S to PWC.  */
UWC_EXTERN_C size_t u_mbrtowc (wchar_t * __restrict __pwc,
		       __const char * __restrict __s, size_t __n,
		       mbstate_t * __p);

/* Write multibyte representation of wide character WC to S.  */
UWC_EXTERN_C size_t u_wcrtomb (char * __restrict __s, wchar_t __wc,
		       mbstate_t * __restrict __ps);

/* Return number of bytes in multibyte character pointed to by S.  */
UWC_EXTERN_C size_t u_mbrlen (__const char * __restrict __s, size_t __n,
		      mbstate_t * __restrict __ps);

/* Return the number of wide characters in S.  */
UWC_EXTERN_C size_t u_wcslen (__const wchar_t * __s);


/* Determine number of column positions required for C.  */
UWC_EXTERN_C int u_wcwidth (wchar_t __c);

/* Determine number of column positions required for first N wide
   characters (or fewer if S ends before this) in S.  */
UWC_EXTERN_C int u_wcswidth (__const wchar_t * __s, size_t __n);


/*======================================================================*\
|*			End						*|
\*======================================================================*/
#endif /* UWC_HEADER */

